{% load i18n %}
<!DOCTYPE html>
<html lang="en">

<head>
    <!-- DEFAULTS -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1,shrink-to-fit=no">
    <meta name="theme-color" content="#000000">
    <meta http-equiv="Cache-Control" content="no-cache, no-store, must-revalidate" />
    <meta http-equiv="Pragma" content="no-cache" />
    <meta http-equiv="Expires" content="0" />
    <!-- TITLE -->
    <title>{{WBCORE_CONTEXT.TITLE}}</title>
    <!-- FAVICON -->
    {% if WBCORE_CONTEXT.FAVICON_URL %}
    <link rel="shortcut icon" type="image/png" href="{{WBCORE_CONTEXT.FAVICON_URL}}" />
    {% endif %}
    <!-- FONT -->
    <link href="{{WBCORE_CONTEXT.FONT_URL}}" rel="stylesheet">
    <!-- EXTERNAL CSS FILES -->
    {% for wbcore_css_url in WBCORE_CONTEXT.CSS_URLS %}
    <link href="{{wbcore_css_url}}" rel="stylesheet" />
    {% endfor %}
    <!-- EXTERNAL CDNS -->
    {% for wbcore_cdn_url in WBCORE_CONTEXT.CDN_URLS %}
    <script crossorigin referrerpolicy="origin" src="{{wbcore_cdn_url}}"></script>
    {% endfor %}
    <!-- WBCORE CSS -->
    {% if WBCORE_CONTEXT.CSS_URL %}
    <link href="{{WBCORE_CONTEXT.CSS_URL}}" rel="stylesheet" type="text/css">
    {% endif %}

    {% if WBCORE_CONTEXT.CONFIG_URL %}
    <script type="text/javascript">window.wbcore_config_url = "{{WBCORE_CONTEXT.CONFIG_URL}}"</script>
    {% endif %}

</head>

<body><noscript>{% translate "You need to enable JavaScript to run this app." %}</noscript>
    <div id="root"></div>
    <!-- WBCORE JS -->
    <script>window.idle_time = {{WBCORE_CONTEXT.IDLE_TIME}};</script>
    <script>window.idle_warning_time = {{WBCORE_CONTEXT.IDLE_WARNING_TIME}};</script>
    {% if WBCORE_CONTEXT.JS_URL %}
    <script type="module" src="{{WBCORE_CONTEXT.JS_RUNTIME_URL}}"></script>
    <script type="module" src="{{WBCORE_CONTEXT.JS_URL}}"></script>
    {% endif %}
</body>

</html>
