# Generated by Django 4.1.8 on 2023-04-12 12:56

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("directory", "0001_initial"),
        ("agenda", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="calendaritem",
            name="entities",
            field=models.ManyToManyField(
                related_name="calendar_entities", to="directory.entry", verbose_name="Entities"
            ),
        ),
        migrations.AddIndex(
            model_name="calendaritem",
            index=models.Index(fields=["period"], name="agenda_cale_period_e7e49e_idx"),
        ),
    ]
