# Generated by Django 4.1.8 on 2023-04-26 08:45

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0002_auto_20230414_1553"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="entry",
            name="last_event",
        ),
    ]
