# Generated by Django 5.0.6 on 2024-07-10 11:34

from decimal import Decimal

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("example_app", "0003_change_stadium_capacity"),
    ]

    operations = [
        migrations.AlterField(
            model_name="player",
            name="transfer_value",
            field=models.DecimalField(
                decimal_places=2, default=Decimal("0"), max_digits=10, verbose_name="Market Value"
            ),
        ),
    ]
