# Generated by Django 5.0.3 on 2024-04-24 11:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0004_alter_notification_body"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="endpoint",
            field=models.CharField(blank=True, max_length=2048, null=True),
        ),
    ]
