
<!DOCTYPE html>
<html lang="en">

<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="shortcut icon" type="image/png" href='favicon.ico'/>

	<title>{{ title }}</title>

	<!-- Google font -->
  <link href="https://fonts.googleapis.com/css?family=Roboto:300,500&display=swap" rel="stylesheet">

	<!-- Custom stlylesheet -->
    <style type="text/css">
      *,*:before,*:after{box-sizing:border-box;}
      html,body{min-height:100%;padding:0;margin:0;font-family:'Roboto',sans-serif;font-size:12px;font-weight:300;color:#000;background-color:transparent;-webkit-overflow-scrolling:touch}
      #maintenance {
        position: relative;
        height: 100vh;
      }

      #maintenance .maintenance {
        position: absolute;
        left: 50%;
        top: 50%;
        -webkit-transform: translate(-50%, -50%);
            -ms-transform: translate(-50%, -50%);
                transform: translate(-50%, -50%);
      }

      .maintenance {
        max-width: 500px;
        width: 100%;
        text-align: center;
      }

      .maintenance .maintenance-404 {
        height: 300px;
        position: relative;
        z-index: -1;
      }

      .maintenance .maintenance-404 h1 {
        font-family: 'Montserrat', sans-serif;
        font-size: 120px;
        margin: 0;
        font-weight: 900;
        position: absolute;
        line-height: 1;
        left: 50%;
        -webkit-transform: translateX(-50%);
            -ms-transform: translateX(-50%);
                transform: translateX(-50%);
        background: url('https://stainly-cdn.fra1.digitaloceanspaces.com/media/bg_orange.jpg') no-repeat;
        -webkit-background-clip: text;
        -webkit-text-fill-color: transparent;
        background-size: cover;
        background-position: center;
      }


      .maintenance h2 {
        font-family: 'Montserrat', sans-serif;
        color: #000;
        font-size: 24px;
        font-weight: 700;
        text-transform: uppercase;
        margin-top: 0;
      }

      .maintenance p {
        font-family: 'Montserrat', sans-serif;
        color: #000;
        font-size: 14px;
        font-weight: 400;
        margin-bottom: 20px;
        margin-top: 0px;
      }

      .maintenance a {
        font-family: 'Montserrat', sans-serif;
        font-size: 14px;
        text-decoration: none;
        text-transform: uppercase;
        background: #ff783a;
        display: inline-block;
        padding: 15px 30px;
        border-radius: 40px;
        color: #fff;
        font-weight: 700;
        -webkit-box-shadow: 0px 4px 15px -5px #ff783a;
                box-shadow: 0px 4px 15px -5px #ff783a;
      }


      @media only screen and (max-width: 767px) {
          .maintenance .maintenance-404 {
            height: 142px;
          }
          .maintenance .maintenance-404 h1 {
            font-size: 112px;
          }
      }
    </style>

	<!-- HTML5 shim and Respond.js for IE8 support of HTML5 elements and media queries -->
	<!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
	<!--[if lt IE 9]>
		  <script src="https://oss.maxcdn.com/html5shiv/3.7.3/html5shiv.min.js"></script>
		  <script src="https://oss.maxcdn.com/respond/1.4.2/respond.min.js"></script>
		<![endif]-->

</head>

<body>

	<div id="maintenance">
		<div class="maintenance">
			<div class="maintenance-404">
				<h1>{{ title }}</h1>
			</div>
			<h2>{{ header }}</h2>
			<p>{{ description }}</p>
		</div>
	</div>

</body><!-- This templates was made by Colorlib (https://colorlib.com) -->

</html>
