from .person import SportPersonTitleConfig, PlayerTitleConfig, PlayerTeamTitleConfig
from .sport import SportTitleConfig
from .league import LeagueTitleConfig, LeagueSportTitleConfig
from .stadium import StadiumTitleConfig
from .team import TeamTitleConfig, TeamStadiumTitleConfig
from .role import RoleTitleConfig
from .match import MatchTitleConfig, MatchStadiumTitleConfig, MatchLeagueTitleConfig
from .event import EventTitleConfig, EventTypeSportTitleConfig, EventTypeTitleConfig, PlayerStatisticsChartTitleConfig
from .teamresult import TeamResultsTitleConfig, TeamResultsLeagueTitleConfig
