# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0006_employeremployeerelationship_position_name"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="bankingcontact",
            options={
                "permissions": (("administrate_bankingcontact", "Can Administrate Banking Contact"),),
                "verbose_name": "Banking Contact",
                "verbose_name_plural": "Banking Contacts",
            },
        ),
    ]
