# Generated by Django 4.2.7 on 2024-02-07 10:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("workflow", "0004_alter_userstep_assignee_method"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userstep",
            name="assignee_method",
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name="Assignee Method"),
        ),
    ]
