# Generated by Django 4.2.6 on 2023-11-02 08:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("directory", "0005_entry_salutation"),
    ]

    operations = [
        migrations.AddField(
            model_name="employeremployeerelationship",
            name="position_name",
            field=models.CharField(
                blank=True,
                help_text="If the selected position isn't an exact match, feel free to use this field to provide more details about the position. You can edit this field once the position is confirmed.",
                max_length=128,
                null=True,
                verbose_name="Alternative Position Name",
            ),
        ),
    ]
