# Generated by Django 4.1.9 on 2023-05-30 07:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0003_notificationusertoken_updated"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="body",
            field=models.TextField(blank=True, null=True),
        ),
    ]
