# Generated by Django 4.1.9 on 2023-06-02 06:15

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("agenda", "0006_calendaritem_is_deletable"),
    ]

    operations = [
        migrations.RemoveField(name="endpoint_basename", model_name="calendaritem"),
        migrations.AlterModelOptions(
            name="calendaritem",
            options={
                "permissions": [("administrate_confidential_items", "Can administrate calendar items")],
                "verbose_name": "Calendar Item",
                "verbose_name_plural": "Calendar Items",
            },
        ),
    ]
