# Generated by Django 5.0.6 on 2024-07-09 10:42

from django.db import migrations, models


class Migration(migrations.Migration):
    replaces = [
        ("example_app", "0003_stadium_seating_capacity_stadium_standing_capacity"),
        ("example_app", "0004_remove_stadium_capacity"),
    ]

    dependencies = [
        ("example_app", "0002_sportperson_profile"),
    ]

    operations = [
        migrations.AddField(
            model_name="stadium",
            name="seating_capacity",
            field=models.PositiveIntegerField(default=0, verbose_name="Seating Capacity"),
        ),
        migrations.AddField(
            model_name="stadium",
            name="standing_capacity",
            field=models.PositiveIntegerField(default=0, verbose_name="Standing Capacity"),
        ),
        migrations.RemoveField(
            model_name="stadium",
            name="capacity",
        ),
    ]
