# Generated by Django 4.2.9 on 2024-02-27 13:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("documents", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="documentmodelrelationship",
            name="primary",
            field=models.BooleanField(default=False, verbose_name="Primary"),
        ),
        migrations.AlterUniqueTogether(
            name="documentmodelrelationship",
            unique_together={("document", "content_type", "object_id", "primary")},
        ),
    ]
