# Generated by Django 4.1.10 on 2023-08-30 11:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("workflow", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="step",
            name="step_type",
            field=models.CharField(
                choices=[
                    ("StartStep", "Start Step"),
                    ("UserStep", "User Step"),
                    ("DecisionStep", "Decision Step"),
                    ("SplitStep", "Split Step"),
                    ("JoinStep", "Join Step"),
                    ("EmailStep", "Email Step"),
                    ("ScriptStep", "Script Step"),
                    ("FinishStep", "Finish Step"),
                ],
                default="",
                max_length=64,
                verbose_name="Step Type",
            ),
        ),
    ]
