# Generated by Django 4.1 on 2022-10-04 06:30

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("authentication", "0004_token"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="metadata",
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
